package japflash;

import java.awt.Panel;
import java.util.Vector;
import java.io.*;
import java.awt.*;
import java.util.StringTokenizer;
import java.awt.event.*;

/**
 * Title:        JapFlash
 * Copyright:
 * Parts of this program (Japanese Bitmap Font support) are taken
 * from KanjiFlash by Marshall Ramsey
 * This program is the copyright of Alexander Bergner, however, you have
 * permission to use the source code as long as the programs you create
 * will be made available without charge and you include the programs'
 * source code and credit is given to me and Marshall Ramsey for providing
 * the code you use.
 * You may distribute this program as long as all the files are included
 * in the distribution and it is provided free of charge or for a small
 * materials fee.
 * @author insaneLX
 * @version 1.0
 */


public class FileSelectorPanel extends Panel implements
  FilenameFilter,ItemListener,ActionListener {

  Vector lessonDescriptors = new Vector();
  String pathList;
  List  fileList;
  Choice cfRamChoice;
  TextField directoryField;
  Options o;
  Panel previewPanel;
  Panel dirPanel;
  Panel topPanel;
  Label descriptionLabel;
  Label correctLabel;
  Label lastOpendedLabel;
  Label cardsLabel;
  Button loadButton;
  ItemListener itemListener;
  String[] pathLists;
  String[] pathListNames;
  Frame ownerFrame;

  public FileSelectorPanel(Options o,Frame ownerFrame) {
    this.o = o;
    this.ownerFrame = ownerFrame;

    String token;
    StringTokenizer st = new StringTokenizer(o.directoryList,"|");
    int tokens = st.countTokens();
    pathLists = new String[tokens];
    pathListNames = new String[tokens];
    for (int i=0; i< tokens;i++) {
      token = st.nextToken();
      pathListNames[i] = token.substring(0,token.indexOf("#"));
      pathLists[i] = token.substring(token.indexOf("#")+1,token.length());
    }
    init();
  }

  public void init() {
    setLayout(new BorderLayout());
    cfRamChoice = new Choice ();
    fileList = new List();
    previewPanel = new Panel();
    descriptionLabel = new Label("-");
    correctLabel =  new Label("-");
    lastOpendedLabel = new Label("-");
    cardsLabel = new Label("-");
    loadButton = new Button("Load Lesson");
    directoryField = new TextField();
    topPanel = new Panel();
    dirPanel = new Panel();

    for (int i =0; i < pathListNames.length; i++) {
      cfRamChoice.add(pathListNames[i]);
    }
    cfRamChoice.addItemListener(this);
    fileList.addItemListener(this);
    fileList.addActionListener(this);

    previewPanel.setBackground(Color.lightGray);
    previewPanel.setLayout(new GridLayout(4,2));
    previewPanel.add(new Label("Description:"));
    previewPanel.add(descriptionLabel);
    previewPanel.add(new Label("Cards:"));
    previewPanel.add(cardsLabel);
    previewPanel.add(new Label("Correctness:"));
    previewPanel.add(correctLabel);
    previewPanel.add(new Label("last Opened:"));
    previewPanel.add(lastOpendedLabel);
    dirPanel.setLayout(new GridLayout(2,1));

    topPanel.setLayout(new BorderLayout());
    topPanel.add(previewPanel,BorderLayout.CENTER);
    topPanel.add(loadButton,BorderLayout.SOUTH);
    add(fileList,BorderLayout.CENTER);
    dirPanel.add(cfRamChoice);
    dirPanel.add(directoryField);
    add(dirPanel,BorderLayout.SOUTH);
    add(topPanel,BorderLayout.NORTH);
    loadButton.setEnabled(false);
    loadButton.addActionListener(this);
    if (pathLists.length>0) setPathList(pathLists[0]);

    directoryField.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        setPathList(directoryField.getText());
      };
    });
    addComponentListener(new ComponentAdapter() {
      public void componentShown(ComponentEvent e) {
        setPathList(pathList);
      };
    });
  }

  public void setItemListener(ItemListener il) {
    this.itemListener = il;
  }

  public void itemStateChanged(ItemEvent e) {
    if (e.getSource() == cfRamChoice) {
      for (int i=0;i<pathListNames.length;i++)
        if ( ((String)e.getItem()).equals(pathListNames[i]))
          setPathList(pathLists[i]);
    }
    if (e.getSource() == fileList) {
      if (e.getStateChange() == e.SELECTED) {
        LessonAttributes la =
          ((LessonDescriptor)(lessonDescriptors.elementAt(fileList.getSelectedIndex()))).lessonAttributes;
        descriptionLabel.setText(la.description);
        if (!la.description.equals("Never Opened")) {
          correctLabel.setText(la.correctness+ "%");
          lastOpendedLabel.setText(getTimeDifference(la.lastOpened)+ " ago");
          cardsLabel.setText(la.cards + "cards");
        } else {
          correctLabel.setText("-");
          lastOpendedLabel.setText("-");
          cardsLabel.setText("-");
        }
        loadButton.setEnabled(true);
      }
      if (e.getStateChange() == e.DESELECTED) {
        descriptionLabel.setText("");
        correctLabel.setText("");
        lastOpendedLabel.setText("");
        cardsLabel.setText("");
        loadButton.setEnabled(false);
      }
    }
  };

  public void actionPerformed(ActionEvent e) {
    if ((fileList.getSelectedIndex() != -1)&&(itemListener != null))
      itemListener.itemStateChanged(
        new ItemEvent(fileList,
                      ItemEvent.ITEM_FIRST,
                      lessonDescriptors.elementAt(fileList.getSelectedIndex()),
                      ItemEvent.SELECTED
                      )
        );
  };

  public void setPathList(String pathList) {
    if (pathList == null) return;
    this.pathList = pathList;
    directoryField.setText(pathList);
    loadLessonDescriptors(getAllFiles(pathList));
    descriptionLabel.setText("");
    correctLabel.setText("");
    cardsLabel.setText("");
    lastOpendedLabel.setText("");
    loadButton.setEnabled(false);
    fileList.removeAll();
    for (int i=0; i< lessonDescriptors.size(); i++)
      fileList.add(
        ((LessonDescriptor)lessonDescriptors.elementAt(i)).file.getName());
    validate();
    repaint();
  }

  public Vector getAllFiles(String pathList){
    Vector files = new Vector();
    String[] fileNames;
    File directory;
    StringTokenizer stringTokenizer = new StringTokenizer(pathList,";");
    while (stringTokenizer.hasMoreElements()) {
      directory = new File(stringTokenizer.nextToken());
      fileNames = getFilesNamesInDirectory(directory);
      if (fileNames != null) for (int i = 0; i<fileNames.length; i++) {
        files.addElement(new File(directory.getAbsolutePath(),fileNames[i]));
      }
    }
    return files;
  }

  public void loadLessonDescriptors(Vector fileList) {
    File file;
    LessonAttributes la;
    lessonDescriptors.removeAllElements();
    for (int i=0; i< fileList.size(); i++) {
      file = (File)fileList.elementAt(i);
      la =LessonAttributes.readLessonAttributes(file);
      if (la == null) la = new LessonAttributes("Never Opened",0,0,0,0);
      lessonDescriptors.addElement(new LessonDescriptor(file,la));
    }
  }

  public String[] getFilesNamesInDirectory(File path){
    String[] files = null;
    if (path.isDirectory()) {
      files = path.list(this);
    } else if (accept(null,path.getName())) {
      files = new String[1];
      files[0] = path.getName();
    }
    return files;
  }

  String getTimeDifference(long time) {
    long dt = (System.currentTimeMillis()-time)/(60*1000);
    if (dt < 60) return dt+" minutes";
    dt /= 60;
    if (dt < 60) return dt+" hours";
    dt /= 24;
    if (dt < 7) return dt+" days";
    if (dt < 35) return dt/7+" weeks";
    dt /= 30;
    return dt+" months";
  }

  public boolean accept(File dir,String name) {
   return (
     (name.toLowerCase().endsWith(".utf"))||
     (name.toLowerCase().endsWith(".ut8"))||
     (name.toLowerCase().endsWith(".jis"))||
     (name.toLowerCase().endsWith(".sjs"))
   );
  };
}

class LessonDescriptor {
  LessonAttributes lessonAttributes;
  File file;
  public LessonDescriptor (File f,LessonAttributes la) {
    this.file = f;
    this.lessonAttributes = la;
  }
}
