package japflash;

import java.awt.event.*;
import java.awt.*;
import com.sun.util.*;
import japsupport.*;
import java.io.*;
import java.util.zip.*;

/**
 * Title:        JapFlash
 * Copyright:
 * Parts of this program (Japanese Bitmap Font support) are taken
 * from KanjiFlash by Marshall Ramsey
 * This program is the copyright of Alexander Bergner, however, you have
 * permission to use the source code as long as the programs you create
 * will be made available without charge and you include the programs'
 * source code and credit is given to me and Marshall Ramsey for providing
 * the code you use.
 * You may distribute this program as long as all the files are included
 * in the distribution and it is provided free of charge or for a small
 * materials fee.
 * @author insaneLX
 * @version 1.0
 */


public class FlashCardPrinter implements
                                    PTimerWentOffListener {
  final static int KANJITOENG = 0;
  final static int ENGTOKANJI = 1;
  final static String[] TESTMODENAME = {"K 2 E","E 2 K"};

  private final static int K = 0;
  private final static int R = 1;
  private final static int M = 2;
  private final static int K_R = 3;
  private final static int K_R_M = 4;
  private final static int R_M = 5;
  private final static int K_M = 6;
  private final static int HIDDEN = -1;
  private final static int SHOWING = 0;
  private final static int LIFTANIMATION = 1;
  private final static int HIDEANIMATION = 2;
  private final static int FLASHANIMATION = 3;
  private final static int faceSequence[][][] = {
     {
     {K,K_R,K_R_M},
     {M,R_M,K_R_M}
     },{
     {K,K_M},
     {M,K_M}
     }
    };

  FlashCardEventListener flashCardEventListener;
  Options o;
  FlashCard card;
  private int faceIndex;      // index of visible face of lifted Card
  int animationFrame = -1;
  int displayStatus = HIDDEN;
  PTimerSpec timerSpec = new PTimerSpec();
  PTimer timer;
  Component owner;
  TextPainter readingPainter;
  TextPainter meaningPainter;
  TextPainter kanjiPainter;
  BitMapFont bitMapFont = null;
  Rectangle bounds = new Rectangle();
  Rectangle zoomOutline = new Rectangle();


  public FlashCardPrinter(Options options,Component owner) {
    this.owner = owner;
    this.o = options;
    meaningPainter = new TextPainter(o,
      new Font(o.meaningFontName,o.meaningFontStyle,o.meaningFontSize));

    if ((o.bitmapFontFileName != null)&& !o.bitmapFontFileName.equals("")) {
      try {
          tools.LXToolBox.debugMessage("Trying to load bitmap file:"+o.bitmapFontFileName);
          InputStream largeIn = null;
          FileInputStream largeFileIn =
                      new FileInputStream((new RandomAccessFile(o.bitmapFontFileName,"r")).getFD());
          largeIn = new GZIPInputStream(largeFileIn);
          if (o.bitmapFontFileName.endsWith("k16x16.gz")) {
            bitMapFont=new JISBitMapFont(16, 16, largeIn, 220160,    // k16x16.f00
                                         8, 64, false, true);
          } else if (o.bitmapFontFileName.endsWith("k24x24.gz")) {
            bitMapFont=new JISBitMapFont(24, 24, largeIn, 561808,    // k24x24.f00
                                         8, 64, true, true);
          }else if (o.bitmapFontFileName.endsWith("k48x48.gz")) {
            bitMapFont=new JISBitMapFont(48, 48, largeIn, 2247040,   // k48x48.f00
                                         8, 64, true, true);
          } else {
            tools.LXToolBox.message("Bitmap-font filename not recognised");
            System.exit(0);
          }
          tools.LXToolBox.debugMessage("Successfully loaded Bitmap file.");
      }
      catch (IOException ioe) {
        System.out.println("Error loading bitmap file");
        System.err.println("Exception caught!\n" + ioe);
        ioe.printStackTrace(System.err);
        System.gc();
        System.exit(1);
      }

      readingPainter = new TextPainter(o,bitMapFont,
                                       o.readingFontSize,o.readingFontSize);
      kanjiPainter = new TextPainter(o,bitMapFont,
                                       o.kanjiFontSize,o.kanjiFontSize);
    } else {
      readingPainter = new TextPainter(o,
        new Font(o.readingFontName,o.readingFontStyle,o.readingFontSize));
      kanjiPainter = new TextPainter(o,
        new Font(o.kanjiFontName,o.kanjiFontStyle,o.kanjiFontSize));
    }

    timerSpec.addPTimerWentOffListener(this);
    timerSpec.setRegular(false);
    timerSpec.setAbsolute(false);
    timerSpec.setRepeat(true);
    timerSpec.setTime(1);
    timer = PTimer.getTimer();
  }

  public void showFlashCard(FlashCard flashCard,Rectangle fromRectangle) {
    stopAnimation();
    setFlashCard(flashCard);
    startAnimation(fromRectangle,LIFTANIMATION);
    tools.LXToolBox.debugMessage("showFlashCard");
  }

  public void pushCardBack(Rectangle toRectangle) {
    tools.LXToolBox.debugMessage("pushFlashCard to "+toRectangle);
    card = null;
    tools.LXToolBox.debugMessage("setting card to null");
    stopAnimation();
    if (toRectangle != null) startAnimation(toRectangle,HIDEANIMATION);
  }

  public boolean cardIsLifted() {return (card != null); }

  private void setFlashCard(FlashCard card) {
    this.card = card;
    faceIndex = 0;
    tools.LXToolBox.debugMessage("setFlashcard");
    kanjiPainter.setText(card.kanji,TextPainter.CENTER);
    readingPainter.setText(card.reading,TextPainter.LEFT);
    meaningPainter.setText(card.meaning,TextPainter.LEFT);
  }

  void setFace(int i) {
    tools.LXToolBox.debugMessage("showing face "+i);
    faceIndex = i;
    if (faceIndex >= faceSequence[(card.hasReading())?0:1][o.testMode].length)
      faceIndex = faceSequence[(card.hasReading())?0:1][o.testMode].length-1;
    setTextAreas();
  }

  void setNextFace() {
    setFace(faceIndex+1);
    if (isLastFace()) startAnimation(null,FLASHANIMATION);
    owner.repaint();
  }

  boolean isLastFace() {
    return (faceIndex ==
              faceSequence[(card.hasReading())?0:1][o.testMode].length-1);
  }

  void setTextAreas() {
    int p = o.textPadding;
    int x = bounds.x+p;
    int y = bounds.y+p;
    int w = bounds.width-p-p;
    int h = (bounds.height*9/10)-p-p;
    int faceType = faceSequence[(card.hasReading())?0:1][o.testMode][faceIndex];


    if (faceType == K)  {
      kanjiPainter.setBounds(x,y,w,(h-p)/2);
      kanjiPainter.setVisible(true);
      readingPainter.setVisible(false);
      meaningPainter.setVisible(false);
    }
    if (faceType == M)  {
      meaningPainter.setBounds(x,y+(h-p)/2+p,w,(h-p)/2);
      kanjiPainter.setVisible(false);
      readingPainter.setVisible(false);
      meaningPainter.setVisible(true);
    }
    if (faceType == R)  {
      readingPainter.setBounds(x,y+(h-p)/2+p,w,(h-p)/2);
      kanjiPainter.setVisible(false);
      readingPainter.setVisible(true);
      meaningPainter.setVisible(false);
    }
    if (faceType == K_M) {
      kanjiPainter.setBounds(x,y,w,(h-p)/2);
      meaningPainter.setBounds(x,y+(h-p)/2+p,w,(h-p)/2);
      kanjiPainter.setVisible(true);
      readingPainter.setVisible(false);
      meaningPainter.setVisible(true);
    }
    if (faceType == K_R) {
      kanjiPainter.setBounds(x,y,w,(h-p)/2);
      readingPainter.setBounds(x,y+(h-p)/2+p,(w-p)/2,(h-p)/2);
      kanjiPainter.setVisible(true);
      readingPainter.setVisible(true);
      meaningPainter.setVisible(false);
    }
    if (faceType == R_M) {
      readingPainter.setBounds(x,y+(h-p)/2+p,(w-p)/2,(h-p)/2);
      meaningPainter.setBounds(x+(w-p)/2+p,y+(h-p)/2+p,(w-p)/2,(h-p)/2);
      kanjiPainter.setVisible(false);
      readingPainter.setVisible(true);
      meaningPainter.setVisible(true);
    }
    if (faceType == K_R_M) {
      kanjiPainter.setBounds(x,y,w,(h-p)/2);
      readingPainter.setBounds(x,y+(h-p)/2+p,(w-p)/2,(h-p)/2);
      meaningPainter.setBounds(x+(w-p)/2+p,y+(h-p)/2+p,(w-p)/2,(h-p)/2);
      kanjiPainter.setVisible(true);
      readingPainter.setVisible(true);
      meaningPainter.setVisible(true);
    }
  }


  public Rectangle zoomRectangle(Rectangle from, Rectangle to,float pos) {
    int x = (int)(from.x * pos + to.x* (1-pos));
    int y = (int)(from.y * pos + to.y* (1-pos));
    int w = (int)(from.width * pos + to.width* (1-pos));
    int h = (int)(from.height * pos + to.height* (1-pos));
    return new Rectangle(x,y,w,h);
  }

  public void paint(Graphics g) {
    paintBackground(g);
    if ((displayStatus == SHOWING)||(displayStatus == FLASHANIMATION)) {
      kanjiPainter.drawTextPainter(g);
      readingPainter.drawTextPainter(g);
      meaningPainter.drawTextPainter(g);
    }
  }
  public void paintBackground(Graphics g){
    Rectangle outline;
    int x,y,ex,width,height;
    width = bounds.width-1;
    height = bounds.height/12-1;
    x = bounds.x;
    y = bounds.y+bounds.height-height-1;

    if (displayStatus == HIDDEN) return;
    if (displayStatus == LIFTANIMATION) {
      outline = zoomRectangle(bounds,zoomOutline,(float)animationFrame/o.animationFrames);
    } else if (displayStatus == HIDEANIMATION) {
      outline = zoomRectangle(zoomOutline,bounds,(float)animationFrame/o.animationFrames);
      tools.LXToolBox.debugMessage("printing hide anim");
    } else  {
      outline = bounds;
    }
    g.setColor(o.fCFillColor);
    g.fillRect(outline.x,outline.y,outline.width-1 ,outline.height-1);
    g.setColor(o.fCBorderColor);
    g.drawRect(outline.x,outline.y,outline.width-1 ,outline.height-1);

    if (displayStatus == FLASHANIMATION) {
      g.setColor(Color.green);
      g.fillRect(x,y,width/2,height);
      g.setColor(Color.red);
      g.fillRect(x+width/2,y,width/2,height);
      g.setColor(o.fCBorderColor);
      g.drawRect(x,y,width,height);
    }
    if (displayStatus == SHOWING) {
      for (int i=0; (i < 16); i++) {
        ex = bounds.x + ((i+1)*width)/16;
        if (card.historyIsDefined(i)) {
          g.setColor( card.isCorrect(i)?Color.green:Color.red);
          g.fillRect(x,y,ex-x,height);
        }
        g.setColor(o.fCBorderColor);
        g.drawRect(x,y,ex-x,height);
        x = ex;
      }
    }
  }

  private void startAnimation(Rectangle r, int type) {
    displayStatus = type;
    animationFrame = 0;
    zoomOutline = r;
    switch (displayStatus) {
      case LIFTANIMATION:
      case HIDEANIMATION:  timerSpec.setTime(o.showHideAnimTime); break;
      case FLASHANIMATION: timerSpec.setTime(o.flashAnimTime);break;
    }
    try {timer.scheduleTimerSpec(timerSpec);}
    catch (PTimerScheduleFailedException e) {stopAnimation();}
    tools.LXToolBox.debugMessage("starting animation"+displayStatus);
  }

  private void stopAnimation() {
    tools.LXToolBox.debugMessage("stopping animation"+displayStatus);
    switch (displayStatus) {
      case LIFTANIMATION: setFace(0); displayStatus = SHOWING; break;
      case HIDEANIMATION: displayStatus = HIDDEN; break;
      case FLASHANIMATION: displayStatus = SHOWING; break;
    }
    animationFrame = -1;
    owner.repaint();
    timer.deschedule(timerSpec);
  }

  public void timerWentOff(PTimerWentOffEvent e) {
    tools.LXToolBox.debugMessage("timer went off"+displayStatus);
    if ((displayStatus == FLASHANIMATION)||
        (++animationFrame > o.animationFrames)) stopAnimation();
    owner.repaint();
  }

  public void setBounds(int x,int y,int w, int h) {
    bounds.setBounds(x,y,w,h);
  }

  public void setFlashCardEventListener(FlashCardEventListener fcev) {
    this.flashCardEventListener = fcev;
  }

  public void checkMouseClick (MouseEvent e) {
    if ((displayStatus != SHOWING)&&(displayStatus != FLASHANIMATION)) return;
    if (isLastFace()) {
      if (e.getX()<(bounds.width/2)) {
        flashCardEventListener.gotRight();
      } else {
        flashCardEventListener.gotWrong();
      }
    } else {
        setNextFace();
    }
  }

  public void validate() {
    if (cardIsLifted()) setTextAreas();
  }
}