package japflash;

import java.io.*;
import java.text.ParseException;


/**
 * Title:        JapFlash
 * Copyright:
 * Parts of this program (Japanese Bitmap Font support) are taken
 * from KanjiFlash by Marshall Ramsey
 * This program is the copyright of Alexander Bergner, however, you have
 * permission to use the source code as long as the programs you create
 * will be made available without charge and you include the programs'
 * source code and credit is given to me and Marshall Ramsey for providing
 * the code you use.
 * You may distribute this program as long as all the files are included
 * in the distribution and it is provided free of charge or for a small
 * materials fee.
 * @author insaneLX
 * @version 1.0
 */


public class Lesson {

  LessonAttributes lessonAttributes;
  CardStack cardStack;
  Options o;

  public Lesson(File f,Options o) {
    this.o = o;
    loadLesson(f);
  }

  boolean loadLesson(File file) {
    int currentPile;
    String line;
    StringBuffer buffer;
    FileInputStream fis;
    InputStreamReader isr;
    BufferedReader br;
    lessonAttributes = LessonAttributes.readLessonAttributes(file);
    if (lessonAttributes == null) {
        String fileName = file.getName();
        fileName = fileName.substring(0,fileName.lastIndexOf("."));
        lessonAttributes = new LessonAttributes(fileName,0,0,o.piles,0);
      }
    cardStack = new CardStack(lessonAttributes.piles);
    try {
      buffer = new StringBuffer();
      fis = new FileInputStream(file);
      if ((file.getName().endsWith("ut8"))||file.getName().endsWith("utf")) {
        isr = new InputStreamReader(fis,"UTF8");
        } else if(file.getName().endsWith("jis") || file.getName().endsWith("sjs")) {
        isr = new InputStreamReader(fis,"JISAutoDetect");
      } else {
        isr = new InputStreamReader(fis);
      }
      br = new BufferedReader(isr);
    } catch (IOException e) {
      tools.LXToolBox.debugMessage("Error opening definition file.");
      return false;
    }
    currentPile = lessonAttributes.piles-1;
    FlashCard c = null;
    int lineNum = 0;
     do {
      try {
        line = br.readLine();
        lineNum++;
      } catch (IOException e) {
        tools.LXToolBox.debugMessage("Error reading line "+lineNum+".");
        try {fis.close();} catch (IOException ee) {};
        return false;
      }
      if (line != null) {
        if (line.length() == 0) {
        } else if (line.startsWith("# Pile")) {
          try {
            currentPile = Integer.parseInt(line.substring(7));
            if (currentPile >= lessonAttributes.piles) {
              currentPile = lessonAttributes.piles-1;
              tools.LXToolBox.message("Wrong pile definition: "+ line);
            }
          } catch (NumberFormatException e) {
              tools.LXToolBox.message("Wrong Pile Definintion:"+line);
          }
        } else {
          try {c = new FlashCard(line);} catch (ParseException e) {
             tools.LXToolBox.debugMessage("Dont understand:"+line+" in line "+lineNum+". "+
                             e.getMessage());
            c = null;
          }
          if (c != null) {
            cardStack.putCardToPile(c,currentPile);
            lessonAttributes.cards++;
          }
        }
      }
    } while (line != null);
    try {fis.close();} catch (IOException ee) {};
    if (lessonAttributes.cards == 0) {
      tools.LXToolBox.debugMessage("The file contained no cards.");
      return false;
    }
    lessonAttributes = new LessonAttributes(this);
    return true;
  }

  int findNumberOfPiles(BufferedReader br) {
    String line;
    int pile = -1;
    int maxPile = -1;
    try {
      while ((line = br.readLine()) != null) {
        if (line.startsWith("# Pile")) {
          try {
            pile = Integer.parseInt(line.substring(7));
          } catch (NumberFormatException e) {
              tools.LXToolBox.message("Wrong Pile Definintion:"+line+".");
          }
          maxPile = Math.max(pile,maxPile);
        }
      }
       br.reset();
    }  catch (IOException e) {}
    return maxPile;
  }

  boolean saveLesson(File file)  {
    FlashCard c;
    lessonAttributes = new LessonAttributes(this);
    try {
      StringBuffer buffer = new StringBuffer();
      FileOutputStream fos = new FileOutputStream(file);
      OutputStreamWriter osw = new OutputStreamWriter(fos,"UTF8");
      BufferedWriter bw = new BufferedWriter(osw);

      bw.write(lessonAttributes.toString());
      bw.newLine();
      for (int i = 0; i< cardStack.pile.length; i++) {
        bw.write("# Pile "+i);
        bw.newLine();
        for (int j= 0; j<  cardStack.pile[i].size(); j++) {
          c = (FlashCard)cardStack.pile[i].elementAt(j);
          try {
            bw.write( c.getContent());
            bw.newLine();
          } catch (IOException e) {
            tools.LXToolBox.debugMessage("Error writing line.");
          }
        }
     }
     bw.close();
    } catch (IOException e) {
      System.out.println("Could not save file:"+file.getAbsolutePath()+".");
      return false;
    }
    return true;
  }
}


class LessonAttributes {
  static String DESCRIPTION = "# Description: ";
  static String LASTOPENED = "# Last Opened: ";
  static String PILES = "# Piles: ";
  static String CORRECTNESS = "# Correctness: ";
  static String CARDS = "# Cards: ";

  static int HEADERLINES = 4;

  public String description;
  public long lastOpened;
  public int correctness;
  public int piles;
  public int cards;

  public LessonAttributes () {
    description = null;
  }

  public LessonAttributes (String description,long lastOpened,
                           int correctness,int piles, int cards) {
    this.description = description;
    this.lastOpened = lastOpened;
    this.correctness = correctness;
    this.piles = piles;
    this.cards = cards;
  }

  public LessonAttributes (Lesson lesson) {
    tools.LXToolBox.debugMessage("initialinsing from lesson");
    FlashCard card;
    description = lesson.lessonAttributes.description;
    if (description == null) description = "";
    lastOpened =0;
    correctness = 0;
    cards = 0;
    piles = lesson.cardStack.pile.length;

    for (int i = 0; i < piles; i++) {
      cards += lesson.cardStack.pile[i].size();
      tools.LXToolBox.debugMessage("Cards"+cards);
      for (int c = 0; c < lesson.cardStack.pile[i].size(); c ++) {
        card =(FlashCard)lesson.cardStack.pile[i].elementAt(c);
        if ( card.lastTwoCorrect()) correctness++;
        lastOpened = Math.max(card.lastViewed,lastOpened);
      }
    }
    correctness = (correctness*100)/cards;
  }

  static LessonAttributes readLessonAttributes (File file) {
    String description = null;
    long lastOpened;
    int correctness;
    int piles;
    int cards;

    String line = null;
    FileInputStream fis;
    InputStreamReader isr;
    BufferedReader br;
    try {
      fis = new FileInputStream(file);
      if ((file.getName().endsWith("ut8"))||file.getName().endsWith("utf")) {
        isr = new InputStreamReader(fis,"UTF8");
        } else if(file.getName().endsWith("jis") || file.getName().endsWith("sjs")) {
        isr = new InputStreamReader(fis,"JISAutoDetect");
      } else {
        isr = new InputStreamReader(fis);
      }
      br = new BufferedReader(isr);
    } catch (IOException e) {
      tools.LXToolBox.message("Error opening Lesson file:"+file.getName()+".");
      return null;
    }

      try {
        line = br.readLine();
        if (line.startsWith(DESCRIPTION)) {
          description =line.substring(DESCRIPTION.length(),line.length());
        } else return null;
        line = br.readLine();
        if (line.startsWith(LASTOPENED)) {
          lastOpened = Long.parseLong(line.substring(
                                          LASTOPENED.length(),line.length()));
        } else return null;
        line = br.readLine();
        if (line.startsWith(PILES)) {
          piles = Integer.parseInt(line.substring(
                                           PILES.length(),line.length()));
        } else return null;
        line = br.readLine();
        if (line.startsWith(CORRECTNESS)) {
          correctness = Integer.parseInt(line.substring(
                                          CORRECTNESS.length(),line.length()));
        } else return null;
        line = br.readLine();
        if (line.startsWith(CARDS)) {
          cards = Integer.parseInt(line.substring(
                                          CARDS.length(),line.length()));
        } else return null;
      } catch (Exception e) {
        tools.LXToolBox.message("Error reading line:"+line);
        try {fis.close();} catch (IOException ee) {};
        return null;
      }
      return new LessonAttributes(description,lastOpened,correctness,piles,cards);
    }

    public String toString() {
      return (DESCRIPTION+description+"\n"+
              LASTOPENED+Long.toString(lastOpened)+"\n"+
              PILES+Long.toString(piles)+"\n"+
              CORRECTNESS+Long.toString(correctness)+"\n"+
              CARDS+Long.toString(cards));
    }
  }