package japflash;

import java.awt.*;
import java.awt.event.*;
import java.text.ParseException;
import java.util.Random;
import tools.LXToolBox;
/**
 * Title:        JapFlash
 * Copyright:
 * Parts of this program (Japanese Bitmap Font support) are taken
 * from KanjiFlash by Marshall Ramsey
 * This program is the copyright of Alexander Bergner, however, you have
 * permission to use the source code as long as the programs you create
 * will be made available without charge and you include the programs'
 * source code and credit is given to me and Marshall Ramsey for providing
 * the code you use.
 * You may distribute this program as long as all the files are included
 * in the distribution and it is provided free of charge or for a small
 * materials fee.
 * @author insaneLX
 * @version 1.0
 */


public class StackPanel extends Panel
                        implements ComponentListener,KeyListener,
                                   FocusListener,FlashCardEventListener{
  Graphics offScreenGraphics;
  Image offScreenImage;
  Image backgroundImage = null;
  Options o;

  final static int STANDARD = 0;
  final static int RECENTCARDS = 1;
  final static int PASTCARDS = 2;

  Lesson lesson;
  Color[] stackBackgroundColor;
  Color[] stackColor;
  int[] liftProbabilities;

  FlashCardPrinter flashCardPrinter;
  FlashCard liftedCard;
  Rectangle[] stackContainerRect;

  private Random r = new Random();
  int liftedFromPile;
  int focusedPile;
  boolean hasFocus;

  public StackPanel(Options options) {
    o = options;
    stackContainerRect = new Rectangle[o.piles];
    flashCardPrinter =  new FlashCardPrinter(o,this);
    this.addComponentListener(this);
    this.addFocusListener(this);
    this.addKeyListener(this);
    this.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        checkMouseClick(e);
      }
    });

    flashCardPrinter.setFlashCardEventListener(this);

    for (int i = 0; i<stackContainerRect.length;i++)
      stackContainerRect[i] = new Rectangle();
    setBackground(o.backgroundColor);
    if (o.backgroundImageFileName != null) {
      backgroundImage = Toolkit.getDefaultToolkit().getImage(o.backgroundImageFileName);
    }
    stackBackgroundColor = new Color[o.piles];
    stackColor = new Color[o.piles];
    liftProbabilities = new int[o.piles];
    int p = 100;

    for (int i=0; i< o.piles; i++) {
      float alpha = (1f - (float)i/o.piles);
      stackColor[i] = LXToolBox.blendColor(o.stackStartColor,
                                      o.stackEndColor,
                                      alpha);
      stackBackgroundColor[i] = LXToolBox.blendColor(o.stackBackgroundStartColor,
                                      o.stackBackgroundEndColor,
                                      alpha);
      liftProbabilities[i] =( p = (int)(p *o.liftProbabilityFactor));
    }
  }

  void checkMouseClick(MouseEvent e) {
    if (flashCardPrinter.cardIsLifted()&&(flashCardPrinter.bounds.contains(e.getPoint()))) {
      e.translatePoint(flashCardPrinter.bounds.x,flashCardPrinter.bounds.y);
      flashCardPrinter.checkMouseClick(e);
    } else {checkMouseClickOnPanel(e);}
  }

  void checkMouseClickOnPanel(MouseEvent e) {
    int hitPile = stackContainerRect.length-1;
    if (!hasFocus) requestFocus();
    while  ((hitPile>=0) && !(stackContainerRect[hitPile].contains(e.getPoint()))) {
        hitPile--;
    }
    tools.LXToolBox.debugMessage("mouse Click on Panel, hit pile" + hitPile);

    if (hitPile != -1) {
      setFocusedPile(hitPile);
      if (flashCardPrinter.cardIsLifted()) putCardToFocusedPile(); else liftCardFromFocusedPile();
    } else {
      if (flashCardPrinter.cardIsLifted()) putCardBack(); else liftAnyCard();
    }
  }

  public void update(Graphics g) {
    paint(g);
  }

  public void paint(Graphics g) {
    if (offScreenGraphics == null) makeOffScreenGraphics();
    offScreenGraphics.clearRect(0,0,getSize().width,getSize().height);
    paintBackground(offScreenGraphics);
    flashCardPrinter.paint(offScreenGraphics);
    g.drawImage(offScreenImage,0,0,this);
  }

/*  public void paint(Graphics g) {
    paintBackground(g);
    flashCardPrinter.paint(g);
  }
*/

  public  FlashCard liftCardFromFocusedPile() {
    return liftCardFromPile(focusedPile);
  }

  public  void putCardToFocusedPile() {
    putCardToPile(focusedPile);
  }

  public void setFocusedPile(int p){
    focusedPile = p;
    repaint();
  }

  public void moveFocusLeft() {
    if (focusedPile >0) setFocusedPile(focusedPile -1);
  }
  public void moveFocusRight() {
    if (focusedPile < (lesson.cardStack.pile.length-1)) setFocusedPile(focusedPile +1);
  }

  public FlashCard liftCardFromPile(int p) {
    if (liftedCard != null) return null;
    liftedCard = lesson.cardStack.liftCardFromPile(p);
    liftedFromPile = p;
    if (liftedCard != null) {
      liftedCard.viewed();
      flashCardPrinter.showFlashCard(liftedCard,stackContainerRect[liftedFromPile]);
      repaint();
      return liftedCard;
    } else {return null;}
  }

  public void putCardToPile(int p){         // puts card on top of pile p
    if (liftedCard != null) {
      lesson.cardStack.putCardToPile(liftedCard,p);
      flashCardPrinter.pushCardBack(stackContainerRect[p]);
      liftedCard = null;
      repaint();
    }
  }

  public void putCardBack() {
    putCardToPile(liftedFromPile);
  }

  public void gotRight() {
    if (!cardIsLifted()) return;
    liftedCard.gotRight();
    if ((liftedCard.isCorrect(1))&(liftedFromPile > 0)) liftedFromPile -= 1;
    putCardToPile(liftedFromPile);
    // System.out.println("got right");
  }

  public void gotWrong() {
    if (!cardIsLifted()) return;
    liftedCard.gotWrong();
    if (liftedFromPile < lesson.cardStack.pile.length-1) liftedFromPile += 1;
    putCardToPile(liftedFromPile);
    // System.out.println("got wrong");
  }

  public boolean cardIsLifted(){
    return (liftedCard != null);
  }

  public FlashCard liftAnyCard(){  // lifts a card from pile p,pos
    FlashCard c = null;

    if (o.liftStrategy == STANDARD) {
      float[] weighedProb = new float[o.piles];
      float rnd = r.nextFloat();
      int i;
      float maxProb = -1;
      int chosenPile = -1;

      for (i =0; i< o.piles;i++) {
        weighedProb[i] = r.nextFloat()*liftProbabilities[i];
      }
      while ((maxProb != 0)&&(c == null)) {
        maxProb = 0;
        for (i =0; i< liftProbabilities.length;i++) {
          if (maxProb < weighedProb[i]) {
            maxProb = weighedProb[i];
            chosenPile = i;
          }
        }
        c = liftCardFromPile(chosenPile);
        if (c == null) {weighedProb[chosenPile] = 0;}
      }
      if (c == null) {// System.out.println("Could not find a card on the stack");
      }
    }
    if (o.liftStrategy == RECENTCARDS) {}
    if (o.liftStrategy == PASTCARDS) {}
    return c;
  }

  void paintBackground(Graphics g) {
    if (backgroundImage != null)
      offScreenGraphics.drawImage(backgroundImage,
            0,0,getSize().width,getSize().height,this);
    if (lesson != null) {
      if (lesson.cardStack.isEmpty() && !cardIsLifted()) {
      } else {
        int ps;
        int psmax = 60;
        int psmod = 50;
        int x;
        int containerHeight = getBounds().height/2;
        int y = getBounds().height-containerHeight-5;
        float w = (float)super.getBounds().width/(1+lesson.cardStack.pile.length*3);
        int height;
        int width = (int)w*2;

        for (int i = 0; i<lesson.cardStack.pile.length; i++) {
          x = (int)(w*(3*i+1));
          ps =  lesson.cardStack.pile[i].size();

          // when the card is on its way back, the pile it is being put onto must
          // is drawn shorter, because the card is technicall already on pile[target]
  //        if ((i == targetPile)&(frame > 0)& !st.cardIsLifted()& ps >0) {ps -= 1;}
          ps = ( (ps>psmax)?( (ps-psmax)%psmod)+(psmax-psmod):ps);
          height = ((containerHeight-4)*(ps))/psmax;
          g.setColor(stackBackgroundColor[i]);
          g.fill3DRect(x,y,width,containerHeight,true);       // Stack container rectangle
          if (focusedPile == i) {
            g.setColor((hasFocus)?Color.black:Color.gray);
            g.draw3DRect(x,y,width,containerHeight,true);
          }
          stackContainerRect[i].setBounds(x,y,width,containerHeight);
          g.setColor(stackColor[i]);
          g.fillRect(x+1,y+containerHeight-height-1,width-2,height);       // Stack rectangle
        }
      }
    }
  }
  public void componentResized(ComponentEvent e) {
    flashCardPrinter.setBounds((int)(o.cardInsetsH),
                  (int)(o.cardInsetTop),
                  (int)(getSize().width-2*o.cardInsetsH),
                  (int)(getSize().height-o.cardInsetTop-o.cardInsetBottom));
    makeOffScreenGraphics();
    if (backgroundImage != null)
      backgroundImage = backgroundImage.getScaledInstance(
                        getSize().width,getSize().height,Image.SCALE_FAST);
    flashCardPrinter.validate();
    repaint();
  }

  void makeOffScreenGraphics() {
    offScreenImage = this.createImage(getSize().width,getSize().height);
    offScreenGraphics = offScreenImage.getGraphics();
  }

  public void componentMoved(ComponentEvent e) {}
  public void componentShown(ComponentEvent e) {requestFocus();}
  public void componentHidden(ComponentEvent e) {}
  public void keyPressed(KeyEvent keyEvent){
    int keyCode = keyEvent.getKeyCode();
    switch (keyCode) {
      case KeyEvent.VK_LEFT: if (flashCardPrinter.cardIsLifted()&&
                                 flashCardPrinter.isLastFace()) {
                              gotRight();
                            } else {
                              moveFocusLeft();
                            }; break;
      case KeyEvent.VK_RIGHT: if (flashCardPrinter.cardIsLifted()&&
                                  flashCardPrinter.isLastFace()) {
                               gotWrong();
                            } else {
                              moveFocusRight();
                            }; break;
      case KeyEvent.VK_DOWN:
        if (cardIsLifted()) {
          if (!flashCardPrinter.isLastFace()) flashCardPrinter.setNextFace();
          else putCardToFocusedPile();
          repaint();
        } else {
          liftCardFromFocusedPile();
        } break;
      case KeyEvent.VK_ESCAPE: putCardToFocusedPile(); break;
//      case KeyEvent.VK_DELETE: if (isLastFace()) gotWrong(); break;
//      case KeyEvent.VK_INSERT: if (isLastFace()) gotRight(); break;
      case KeyEvent.VK_UP:
      case KeyEvent.VK_ENTER:
        if (cardIsLifted()) {
          if (!flashCardPrinter.isLastFace()) {
            flashCardPrinter.setNextFace(); repaint();
          } else {
          //  o.statusMessage("Right or Wrong?");
          }
        } else {
          liftAnyCard();
        }
      break;
    }
    keyEvent.consume();
  };

  public void keyReleased(KeyEvent keyEvent) {};
  public void keyTyped(KeyEvent keyEvent){}
  public void focusGained(FocusEvent fE) {
//    o.statusMessage("Focus gained");
    hasFocus = true;
    repaint();
  };
  public void focusLost(FocusEvent fE) {
//    o.statusMessage("Focus lost");
    hasFocus = false;
    repaint();
  };

  public boolean isFocusTraversable() {return true;};

  public void requestInfo(Character c) {}

}