package japflash;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.ImageObserver;
import japsupport.*;
import java.io.*;
import java.util.Vector;
import tools.LXToolBox;

/**
 * Title:        JapFlash
 * Copyright:
 * Parts of this program (Japanese Bitmap Font support) are taken
 * from KanjiFlash by Marshall Ramsey
 * This program is the copyright of Alexander Bergner, however, you have
 * permission to use the source code as long as the programs you create
 * will be made available without charge and you include the programs'
 * source code and credit is given to me and Marshall Ramsey for providing
 * the code you use.
 * You may distribute this program as long as all the files are included
 * in the distribution and it is provided free of charge or for a small
 * materials fee.
 * @author insaneLX
 * @version 1.0
 */

public class TextPainter implements ImageObserver {

  static final int CENTER = 1;
  static final int LEFT = 0;

  Options o;
  boolean useBitmapFonts;
  BitMapFont bitMapFont;
  int bMFWidth;
  int bMFHeight;
  private String text;
  int alignment;
  Rectangle bounds = new Rectangle(0,0,0,0);
  boolean visible = false;

  Font ttFont;
  FontMetrics currentMetrics = null;

  public TextPainter(Options o,Font ttFont) {
    this.o = o;
    setTTFont(ttFont);
  }

  public TextPainter(Options o,BitMapFont bitMapFont, int width, int height ) {
    this.o = o;
    setFont(bitMapFont,width,height);
  }

  public void setBounds(int x,int y,int w, int h) {
    bounds.setBounds(x,y,w,h);
  }

  public void setVisible(boolean visible) {
    this.visible = visible;
  }

  void setFont(BitMapFont bmf,int width,int height) {
    this.bitMapFont = bmf;
    useBitmapFonts = true;
    setBitMapFontSize(width,height);
  }
  void setTTFont(Font ttf) {
    ttFont = ttf;
    useBitmapFonts = false;
  }

  void setBitMapFontSize(int w,int h) {
    this.bMFWidth = w; this.bMFHeight = h;
  }

  public void setText(String text, int alignment) {
    this.alignment = alignment;
    this.text = text;
    tools.LXToolBox.debugMessage("set text to "+text+ " aligned to:"+alignment);
  }

  public void drawTextPainter(Graphics g) {
    Rectangle oldClip = g.getClipBounds();
    g.setColor(Color.black);
    if (!visible) return;
    String txt = text.trim();
    if ((txt == null)||(txt.length()==0)) return;
    g.setClip(bounds);

    int fontHeight;
    int fontAscent;
    String line;
    int pos;
    int carety = bounds.y;
    int caretx = bounds.x;
    int w =  bounds.width;
    int h = bounds.height;
    boolean testLineFits;
    boolean evenLine = false;

    if (useBitmapFonts) {
      fontAscent =  fontHeight = bMFHeight;
    }
    else {
      fontHeight = g.getFontMetrics().getHeight();
      fontAscent = g.getFontMetrics().getAscent();
    }

    if (alignment == CENTER) caretx += w/2;
    if (alignment == CENTER) tools.LXToolBox.debugMessage(text +" is centered");

    while ((line = getNextLine(txt)).length()>0) {
      if (line.equals(txt)) {
        txt = "";
      } else {
        txt = txt.substring(line.length()+1,txt.length());
      }
      pos = 0;
      while (pos < line.length()) {
        line = line.substring(pos,line.length()).trim();                        // cut off the charcters already printed
        pos = getNextFittingAdvance(g,line,w);
        if (pos == -1) pos = line.length();
        carety += fontAscent;
        drawString(g, line.substring(0,pos),
          (alignment == CENTER)?
            (caretx-stringWidth(g,line.substring(0,pos))/2):caretx,
          carety);
        carety += fontHeight-fontAscent;
      }
    }
    g.setClip(oldClip);
  }




// gets the wrappable substring longer than afterPos or the whole thing if
//   the substring contains no space
  String getNextToken(String t,int afterPos) {
    int pos = t.indexOf(" ",afterPos+1);
    if (pos == -1 ) return t;
    return t.substring(0,pos);
  }

// returns the first substring up to a forward slash or the whole thing
  String getNextLine(String t) {
    return (t.indexOf("/") != -1)?(t.substring(0,t.indexOf("/"))):t;
  }

  int getNextFittingAdvance(Graphics g,String t,int x) {
    String test;
    String fittingTest;
    boolean testFits;

    fittingTest = null;
    test = "";
    do {
      test = getNextToken(t,test.length());
      testFits = (stringWidth(g,test) <= x);
      if (testFits) fittingTest = test;
    } while ( testFits && (!test.equals(t)));

    // if the shortest token doesntt fit, wrap at character boundaries
    if (fittingTest == null) {
      test = "";
      do {
        test = t.substring(0,test.length()+1);
        testFits = (stringWidth(g,test) <= x);
        if (testFits) fittingTest = test;
      } while ( testFits && !t.equals(test));
      if (fittingTest == null) return -1;
    }
    return fittingTest.length();
  }

  int stringWidth(Graphics g,String w) {
    return (useBitmapFonts?
            (int)(bMFWidth*w.length()*1.05)
            :
            g.getFontMetrics().stringWidth(w) );
  }

  private void drawString(Graphics g,String string,int x, int y) {

    if (useBitmapFonts) {
       drawBitMapString(g, string,x,y);
    } else {
      g.drawString(string,x,y);
    }
  }

  void drawBitMapString(Graphics g, String txt,int x,int y) {

    try {
      final int BYTESPERCHAR = 2;
      sun.io.CharToByteISO2022JP converter = new sun.io.CharToByteISO2022JP();
      byte ba[] = new byte[txt.length()*BYTESPERCHAR+128];
      converter.convert(txt.toCharArray(),0,txt.length(),ba,0,ba.length);
      ByteArrayInputStream byteIn = new ByteArrayInputStream(ba);

      JISInputStream in = new JISInputStream(byteIn);
      Vector v = new Vector(txt.length());
      try {
	while(true) {
	  byte[] bytes = in.readBytes();
	  if (bytes != null)
	    v.addElement(bytes);
	  else
	    break;
	}
      }
      catch (NullPointerException npe) {
	// do nothing -- signals end of text
      }
      int totalBytes = 0;
      for (int i = 0; i < v.size(); i++) {
	totalBytes += ((byte[])(v.elementAt(i))).length;
      }
      byte[] bytes = new byte[totalBytes];
      int curPos = 0;
      for (int i = 0; i < v.size(); i++) {
	byte[] buff = (byte[])v.elementAt(i);
	System.arraycopy(buff, 0, bytes, curPos, buff.length);
	curPos += buff.length;
      }
      printChars(g, bytes,x,y);
    }
    catch(IOException ioe) {
      ioe.printStackTrace();
    }
  }

  public void printChars(Graphics g, byte bytes[],int x,int y) {
    int bytesPerChar =2;
    tools.LXToolBox.debugMessage("Printing bitmap string"+bytes.toString());
    try {
      Image charImage;
      int numChars = bytes.length / bytesPerChar;
      byte buf[] = new byte[bytesPerChar];
      for (int i=0; i<numChars; i++) {
	System.arraycopy(bytes, i*bytesPerChar, buf, 0, bytesPerChar);
        charImage = bitMapFont.getChar(buf);
	g.drawImage(
          charImage,x,y-bMFHeight,bMFWidth,bMFHeight,this
        );
        x += (int)(bMFWidth*1.05);
      }
    }
    catch (NullPointerException npe) {
    }
  }

  public boolean imageUpdate(Image i,int flags,int x,int y,int w,int h) {
    return((flags | ImageObserver.ALLBITS) !=0 );
  }

/*  public void update(Graphics g) {
    paint(g);
    tools.LXToolBox.debugMessage("updated text painter "+text);
  }

  public void paint(Graphics g) {
    if (offScreenGraphics == null) updateOffScreenImage();
    offScreenGraphics.clearRect(0,0,getSize().width,getSize().height);
    drawTextPainter(offScreenGraphics);
    g.drawImage(offScreenImage,0,0,this);
  }

  public void updateOffScreenImage() {
    offScreenImage = this.createImage(getSize().width,getSize().height);
    offScreenGraphics = offScreenImage.getGraphics();
  }

*/


}