package japsupport;

import java.io.*;
import java.awt.*;
import java.util.*;
import java.awt.image.ImageObserver;
import java.awt.image.MemoryImageSource;

/**
 * Title:        JapFlash
 * Copyright:
 * Parts of this program (Japanese Bitmap Font support) are taken
 * from KanjiFlash by Marshall Ramsey
 * This program is the copyright of Alexander Bergner, however, you have
 * permission to use the source code as long as the programs you create
 * will be made available without charge and you include the programs'
 * source code and credit is given to me and Marshall Ramsey for providing
 * the code you use.
 * You may distribute this program as long as all the files are included
 * in the distribution and it is provided free of charge or for a small
 * materials fee.
 * @author insaneLX
 * @version 1.0
 */

public class JISBitMapFont extends Canvas implements BitMapFont {
  int w;
  int h;
  Hashtable imageCache;
  int maxCache;
  byte[] bitMaps;
  int pixelsPerByte;
  int headerLength; // length of the font header information
  boolean holes;   // true if the font file has empty areas that should be skipped
  boolean reversed; // true if foreground a bg colors should be reversed

  public JISBitMapFont(int width, int height, InputStream in, int fileSize,
		       int pixelsPerByte, int headerLength, boolean holes,
		       boolean reversed) throws IOException {
    imageCache = new Hashtable();
    maxCache = 300;
    this.pixelsPerByte = pixelsPerByte;
    this.headerLength = headerLength;
    this.holes = holes;
    this.reversed = reversed;
    w = width;
    h = height;
    readBitmaps(in, fileSize);
  }

  public JISBitMapFont(int width, int height, InputStream in, int fileSize,
		       int pixelsPerByte, int headerLength, boolean holes,
		       boolean reversed, int maxCache) throws IOException {
    imageCache = new Hashtable();
    this.maxCache = maxCache;
    this.pixelsPerByte = pixelsPerByte;
    this.headerLength = headerLength;
    this.holes = holes;
    this.reversed = reversed;
    w = width;
    h = height;
    readBitmaps(in, fileSize);
  }

  public int getHeight() { return h; }

  public int getWidth() { return w; }

  public Image getChar(int charNum) {
    Integer key = new Integer(charNum);
    Image image;
    if (imageCache.containsKey(key)) {
      image = (Image) imageCache.get(key);
    }
    else {
      int offset = charNum * (w / pixelsPerByte) * h + headerLength;
      BitMapImageProducer prod = new BitMapImageProducer(bitMaps, offset,w,h,8,reversed);
      image = createImage(prod);
      if (imageCache.size() < maxCache)
	imageCache.put(key, image);
      else {
	Enumeration keys = imageCache.keys();
        while (keys.hasMoreElements()) {
          Integer k = (Integer)keys.nextElement();
          ((Image)imageCache.get(k)).flush();
          imageCache.remove(k);
        }
        imageCache.put(key, image);

        //	if (keys.hasMoreElements()) {
//	  imageCache.remove(keys.nextElement());
//	  imageCache.put(key, image);
//	}
      }
    }
    return image;
  }

  public Image getChar(byte bytes[]) {
    Image image = null;
    int index = (int)JIS.Jis2Index(bytes, holes);
    if (index != -1)
      image = getChar(index);
    return image;
  }

  public void readBitmaps(InputStream in, int size) throws IOException {
    DataInputStream dataIn = new DataInputStream(in);
    bitMaps = new byte[size];
    dataIn.readFully(bitMaps);
  }

  public boolean imageUpdate(Image img, int infoflags, int x, int y,
			     int width, int height) {
    if ((infoflags & ImageObserver.ALLBITS) != 0) {
      // repaint();
    }
    return true;
  }
}

