package tools;

/**
 * Title:        JapFlash
 * Copyright:
 * Parts of this program (Japanese Bitmap Font support) are taken
 * from KanjiFlash by Marshall Ramsey
 * This program is the copyright of Alexander Bergner, however, you have
 * permission to use the source code as long as the programs you create
 * will be made available without charge and you include the programs'
 * source code and credit is given to me and Marshall Ramsey for providing
 * the code you use.
 * You may distribute this program as long as all the files are included
 * in the distribution and it is provided free of charge or for a small
 * materials fee.
 * @author insaneLX
 * @version 1.0
 */

import java.awt.Font;
import java.awt.Color;
import java.awt.Label;
import java.io.*;
import java.util.*;
import java.lang.reflect.Field;
import java.text.ParseException;
import java.lang.Integer;

public class FieldInitialiser {



  public FieldInitialiser() {
  }

  public static void loadFields(Object o,String fileName) throws IOException {
    Properties p = new Properties();
    String variableName = "";
    p.load(new FileInputStream(fileName));
    Enumeration variableNames = p.propertyNames();
    while (variableNames.hasMoreElements()) {
      try {variableName = (String)variableNames.nextElement();}
      catch (NoSuchElementException e) {
        throw (new IOException("Class:"+o.getClass().getName()+" does not"+
        " define a field called:"+ variableName));
      };
      setValue(o,variableName,p.getProperty(variableName));
    }
  }

  public static String dumpFields(Object o) {
    Class c = o.getClass();
    Field[] fields;
    Object variable;
    StringBuffer output = new StringBuffer();
    output.append("Class name:"+c.getName()+"\n");
    try {
      fields = c.getFields();
      for (int i =0;i< fields.length; i++) {
        System.out.println(fields[i].getType().getName());
        output.append("Type:"+fields[i].getType().getName());
        output.append(" Name:"+fields[i].getName()+ "=" +
                               fields[i].get(o).toString()+"\n");
      }
    } catch (Exception e){ System.out.println("Error:"+e.getMessage());
    e.printStackTrace();
    return null;}
    System.out.print(output.toString());
    return output.toString();
  };

  public static void saveFields(Object o,String fileName,String header) throws IOException {
    if (o == null)
      throw new IOException("Object to save is Null.");
    if ((fileName == null)||(fileName.length() == 0))
      throw new IOException("No file name specified.");

    Properties p = new Properties();
    Class c = o.getClass();
    Field[] fields;
    Object variable;
    String valueString;
    try {
      fields = c.getFields();
      for (int i =0;i< fields.length; i++) {
        variable = fields[i].get(o);
//        System.out.println("saving"+fields[i].getName());

        if (variable instanceof Integer)  {
          valueString = writeInteger((Integer)variable);
        } else if (variable instanceof Float)  {
          valueString = writeFloat((Float)variable);
        } else if (variable instanceof Color)  {
          valueString = writeColor((Color)variable);
        } else if (variable instanceof String)  {
          valueString = writeString((String)variable);
        } else if (variable instanceof Boolean)  {
          valueString = variable.toString();
        } else {
          throw new IOException("Error when saving options file: Dont not understand "+
                   "class "+variable.getClass().getName()+ " of field called:"+
                   fields[i].getName());
        }
        p.put(fields[i].getName(),valueString);
      }
      p.save(new FileOutputStream(fileName),header);
    } catch (Exception e){
      e.printStackTrace();
      throw new IOException("Exception when saving options file:"+e.getMessage());
    }
    return;
  };

  static void setValue(Object o,String varName,String valueString) throws IOException {
    Class c = o.getClass();
    Field field;
    Object variable;
    try {
      field = c.getField(varName); }
    catch (Exception e) {
      throw (new IOException("Did not find a field called "+varName+ " in" +
     c.getName()));
    }
    try {
      variable = field.get(o); }
    catch (Exception e){
      throw (new IOException("Did not find a field called "+varName+ " in" +
     c.getName()));
    }
    valueString = valueString.trim();
    try{
      if (variable instanceof Integer)  {
        field.set(o,parseInteger(valueString));
      } else if (variable instanceof Float)  {
        field.set(o,parseFloat(valueString));
      } else if (variable instanceof Color)  {
        field.set(o,parseColor(valueString));
      } else if (variable instanceof String)  {
        field.set(o,parseString(valueString));
      } else if (variable instanceof Boolean)  {
        field.set(o,parseBoolean(valueString));
      } else {
        throw (new IOException("Unknow object type:"+variable.getClass().getName()+
                         " defined by "+varName));
      }

    } catch (Exception e){
      e.printStackTrace();
      throw (new IOException("Could not parse \""+valueString+
        "\" for class "+ variable.getClass().getName()));
    }
  }

  static Color parseColor(String colorDef) throws NumberFormatException{
    int r,g,b;
    r = Integer.parseInt(colorDef.substring(0,colorDef.indexOf(",")));
    colorDef = colorDef.substring(colorDef.indexOf(",")+1);
    g = Integer.parseInt(colorDef.substring(0,colorDef.indexOf(",")));
    colorDef = colorDef.substring(colorDef.indexOf(",")+1);
    b = Integer.parseInt(colorDef);
    return new Color(r,g,b);
  }
  static String writeColor(Color c) {
    return ""+c.getRed()+","+c.getGreen()+","+c.getBlue();
  }

  static Boolean parseBoolean(String booleanDef) throws ParseException{
    if (booleanDef.equals("true")) {
      return new Boolean(true);
    }
    if (booleanDef.equals("false")) {
      return new Boolean(false);
    }
    throw (new ParseException("Could not interpret\""+booleanDef+"\" as Boolean",0));
  }
  static String writeBoolean(Boolean b){
    return (b.booleanValue()?"true":"false");
  }

  static Integer parseInteger(String integerDef) throws NumberFormatException {
    return new Integer(Integer.parseInt(integerDef));
  }
  static String writeInteger(Integer i) {
    return i.toString();
  }
  static Float parseFloat(String floatDef) throws NumberFormatException {
    return Float.valueOf(floatDef);
  }
  static String writeFloat(Float f) {
    return f.toString();
  }
  static String parseString(String stringDef) {
    return new String(stringDef);
  }
  static String writeString(String string) {
    return string.toString();
  }
}